/*
  作成者: ihara789
  作成日: 2023/12/01
  更新日: 2023/12/14
*/

// --------------------------------------------

// filed
let playfield_width = 600;
let playfield_height = 400;
let playfield_color = '#2c4f54'

// player
const player_width = 50;
const player_height = 50;
let player_x = (playfield_width - player_width) / 2;
let player_y = (playfield_height - player_width) / 2;
let player_color = '#80aba9'
let player_moving_key = 3.5;
// let player_moving_mouse = 1.5;

const canvas = document.querySelector('#playfield');

// Field
class Playfield {
    constructor(canvas, x, y, width, height, color) {
        this.canvas = canvas;
        this.x = x;
        this.y = y;
        this.width = width;
        this.height = height;
        this.color = color;
        this.canvas.width = this.width;
        this.canvas.height = this.height;
    }
    draw() {
        const ctx = this.canvas.getContext('2d');
        ctx.fillStyle = this.color;
        ctx.fillRect(this.x, this.y, this.width, this.height);
    }
}

// player
class Player {
    constructor(canvas, x, y, width, height) {
        this.canvas = canvas;
        this.p_x = x;
        this.p_y = y;
        this.p_w = width;
        this.p_h = height;
        this.p_c = player_color;
    }
    draw() {
        const ctx = this.canvas.getContext('2d');
        ctx.beginPath();
        ctx.rect(this.p_x, this.p_y, this.p_w, this.p_h);
        ctx.fillStyle = this.p_c;
        ctx.fill();
        ctx.closePath();
    }
    move(dx, dy) {
        this.p_x += dx;
        this.p_y += dy;
        this.draw();
    }
}


//---------------------- control--------------------------

// Field | Rendering
const playfield = new Playfield(canvas, 0, 0, playfield_width, playfield_height, playfield_color);
playfield.draw();

// player | Rendering
const player = new Player(canvas, player_x, player_y, player_width, player_height);

const interval = setInterval(() => {
    playfield.draw();
    player.draw();
}, 1000 / 60);

//---------------------- control--------------------------

// キーボードのキーを押し続けたときに一瞬の停止が発生するのは、キーボードの「リピート」動作によるものです。
// これは、キーを押し続けたときに、最初のキー入力と2回目のキー入力の間にわずかな遅延（リピート遅延）があり、
// その後は一定の速度（リピート速度）でキー入力が続くという動作です。

// この動作は、オペレーティングシステムレベルで制御されているため、
// JavaScriptのkeydownイベントリスナーでは直接制御することはできません。
// しかし、代替の解決策として、キーの押下と放出を追跡する変数を使用して、
// キーが押されている間は特定のアクションを続けるという方法があります。

let keys = {};

document.addEventListener('keydown', (event) => {
    keys[event.code] = true;
});

document.addEventListener('keyup', (event) => {
    keys[event.code] = false;
});

function key_input_Loop() {
    if (keys['ArrowUp']) {
        console.log('ArrowUp');
        player.move(0, - player_moving_key);
    }
    if (keys['ArrowDown']) {
        console.log('ArrowDown');
        player.move(0, player_moving_key);
    }
    if (keys['ArrowLeft']) {
        console.log('ArrowLeft');
        player.move(- player_moving_key, 0);
    }
    if (keys['ArrowRight']) {
        console.log('ArrowRight');
        player.move(player_moving_key, 0);
    }
    if (keys['Space']) {
        console.log('Space');
    }

    player.draw();

    requestAnimationFrame(key_input_Loop);
}

key_input_Loop();