/*
  作成者: ihara789
  作成日: 2023/12/01
  更新日: 2023/12/14
*/

// --------------------------------------------

// filed
let playfield_width = 600;
let playfield_height = 400;
let playfield_color = '#2c4f54'

// player
const player_width = 50;
const player_height = 50;
let player_x = (playfield_width - player_width) / 2;
let player_y = (playfield_height - player_width) / 2;
let player_color = '#80aba9';
let player_moving_key = 3.5;
let player_moving_mouse = 100; // mouseMode = 2 ; only

// target
let ball_color = '#fff352'
// bullet
let bullet_color = '#F39800';

// other configuration
let frame_duration = 10;
let playing = true;
let mouseMode = 3; // (1 Not used , 2 Follow , 3 Overlap)

// canvas ------------
const canvas = document.querySelector('#playfield');

// requestAnimationFrame Config ------------
class DeltaTimeCalculator {
    constructor() {
        this.lastTime = Date.now();
    }
    getDeltaTime() {
        const now = Date.now();
        // 経過時間を秒単位に調整
        const deltaTime = (now - this.lastTime) / 1000;
        this.lastTime = now;
        return deltaTime;
    }
}
const deltaTimeCalculator = new DeltaTimeCalculator();

// Field ---------------------
class Playfield {
    constructor(x, y, width, height, color) {
        this.canvas = canvas;
        this.x = x;
        this.y = y;
        this.width = width;
        this.height = height;
        this.color = color;
        this.canvas.width = this.width;
        this.canvas.height = this.height;
    }
    draw() {
        const ctx = this.canvas.getContext('2d');
        ctx.fillStyle = this.color;
        ctx.fillRect(this.x, this.y, this.width, this.height);
    }
}
// field | Rendering
const playfield = new Playfield(0, 0, playfield_width, playfield_height, playfield_color);
playfield.draw();

// Player ---------------------
class Player {
    constructor(x, y, width, height) {
        this.canvas = canvas;
        this.p_x = x;
        this.p_y = y;
        this.p_w = width;
        this.p_h = height;
        this.p_c = player_color;
    }
    draw() {
        const ctx = this.canvas.getContext('2d');
        ctx.beginPath();
        ctx.rect(this.p_x, this.p_y, this.p_w, this.p_h);
        ctx.fillStyle = this.p_c;
        ctx.fill();
        ctx.closePath();
    }
}
// player | Rendering
const player = new Player(player_x, player_y, player_width, player_height);

// Control ---------------------
class Control {
    constructor() {
        this.keys = {};
        this.mouse_x = 0;
        this.mouse_y = 0;

        document.addEventListener('keydown', (event) => {
            this.keys[event.code] = true;
        });
        document.addEventListener('keyup', (event) => {
            this.keys[event.code] = false;
        });

        document.addEventListener("mousemove", (event) => {
            this.updateMousePosition(event);
        });
    }
    updateMousePosition(event) {
        // Mouse location (window)
        let playfield_rect = canvas.getBoundingClientRect();
        this.mouse_x = event.clientX;
        this.mouse_y = event.clientY;
        // Mouse location (Playfield Player-Center)
        this.mouse_field_x = this.mouse_x - player.p_w / 2 - playfield_rect.left;
        this.mouse_field_y = this.mouse_y - player.p_h / 2 - playfield_rect.top;
    }
    key() {
        if (this.keys['ArrowUp']) {
            // console.log('ArrowUp');
            this.update(0, -player_moving_key);
        }
        if (this.keys['ArrowDown']) {
            // console.log('ArrowDown');
            this.update(0, player_moving_key);
        }
        if (this.keys['ArrowLeft']) {
            // console.log('ArrowLeft');
            this.update(-player_moving_key, 0);
        }
        if (this.keys['ArrowRight']) {
            // console.log('ArrowRight');
            this.update(player_moving_key, 0);
        }
        if (this.keys['Space']) {
            console.log('Space');
        }
    }
    mouseFollow(deltaTime) {
        if (
            // マウスの反応範囲をfield内に制限
            this.mouse_field_x >= -player.p_w / 2 &&
            this.mouse_field_x <= playfield.width - player.p_w / 2 &&
            this.mouse_field_y >= -player.p_h / 2 &&
            this.mouse_field_y <= playfield.height - player.p_h / 2
        ) {
            // filed超過時の補正
            if (this.mouse_field_x <= 0) {
                this.mouse_field_x = 0;
            } else if (this.mouse_field_x >= playfield.width - player.p_h) {
                this.mouse_field_x = playfield.width - player.p_w / 2;
            };
            if (this.mouse_field_y <= 0) {
                this.mouse_field_y = 0;
            } else if (this.mouse_field_y >= playfield.height - player.p_h) {
                this.mouse_field_y = playfield.height - player.p_h / 2;
            };
            // 移動方向の判定
            let distance_to_target_x = Math.abs(this.mouse_field_x - player.p_x);
            let distance_to_target_y = Math.abs(this.mouse_field_y - player.p_y);
            let distance_to_move = player_moving_mouse * deltaTime;
            if (distance_to_target_x >= distance_to_move) {
                if (this.mouse_field_x < player.p_x) {
                    this.update(-player_moving_mouse * deltaTime, 0);
                } else if (this.mouse_field_x > player.p_x) {
                    this.update(player_moving_mouse * deltaTime, 0);
                }
            } else {
                player.p_x = this.mouse_field_x;
            }
            if (distance_to_target_y >= distance_to_move) {
                if (this.mouse_field_y < player.p_y) {
                    this.update(0, -player_moving_mouse * deltaTime);
                } else if (this.mouse_field_y > player.p_y) {
                    this.update(0, player_moving_mouse * deltaTime);
                }
            } else {
                player.p_y = this.mouse_field_y;
            }
        }
    }
    update(dx, dy) {
        if (0 <= player.p_x) {
            player.p_x += dx;
        } else {
            player.p_x = 0;
        }
        if (player.p_x <= playfield_width - player.p_w) {
            player.p_x += dx;
        } else { player.p_x = playfield_width - player.p_w; }
        if (0 <= player.p_y) {
            player.p_y += dy;
        } else {
            player.p_y = 0;
        }
        if (player.p_y <= playfield_height - player.p_h) {
            player.p_y += dy;
        } else {
            player.p_y = playfield_height - player.p_h;
        }
    }
    mouseOverlap() {
        if (
            // マウスの反応範囲をfield内に制限
            this.mouse_field_x >= -player.p_w / 2 &&
            this.mouse_field_x <= playfield.width - player.p_w / 2 &&
            this.mouse_field_y >= -player.p_h / 2 &&
            this.mouse_field_y <= playfield.height - player.p_h / 2
        ) {
            // 超過時の補正
            if (this.mouse_field_x <= 0) {
                this.mouse_field_x = 0;
            } else if (this.mouse_field_x >= playfield.width - player.p_h) {
                this.mouse_field_x = playfield.width - player.p_w;
            }
            if (this.mouse_field_y <= 0) {
                this.mouse_field_y = 0;
            } else if (this.mouse_field_y >= playfield.height - player.p_h) {
                this.mouse_field_y = playfield.height - player.p_h;
            }
            // マウスの座標を代入
            player.p_x = this.mouse_field_x;
            player.p_y = this.mouse_field_y;
        }
    }
}
// control | Rendering
const control = new Control();

// play ---------------------
function playLoop() {
    const deltaTime = deltaTimeCalculator.getDeltaTime();
    playfield.draw();
    player.draw();
    control.key();
    if (mouseMode === 1) {
    } else if (mouseMode === 2) {
        control.mouseFollow(deltaTime);
    } else if (mouseMode === 3) {
        control.mouseOverlap();
    }
    // console.log('DeltaTime:', deltaTime);
    if (playing) {
        requestAnimationFrame(playLoop);
    }
}
playLoop();